/**
 * 基础列表组件
 */
import React from 'react'
import PropTypes from 'prop-types'
import {ListView as AntdListView,PullToRefresh} from 'antd-mobile';
import './styles/ListView.scss'

// 滚动数据
let scrollData = {};
// pageSize
const pageSize = 15;
// 延迟时间
const time = 500;
// 拉动距离限制, 建议小于 200
const damping = 60;
// 控制在滚动过程中，scroll事件被调用的频率
const scrollEventThrottle = 500;
// 调用onEndReached之前的临界值，单位是像素
const onEndReachedThreshold = 100;

class ListView extends React.Component {
  onScroll = (e) => {
    const {id} = this.props;
    scrollData[id] = e.target.scrollTop;
  };

  constructor(props) {
    super(props);
    this.state = {
      size: pageSize,
      refreshing: false
    }
  }

  componentDidMount() {
    const {list, id} = this.props;
    if (!this.props.location) {
      return;
    }
    const {action} = this.props.location;
    if (action === 'PUSH') {
      scrollData[id] = 0;
      this.setState({
        size: pageSize,
      }, () => {
        this.refs[id].scrollTo(0, 0);
      });
    }
    if (action === 'POP') {
      this.setState({
        size: list.length
      }, () => {
        this.refs[id].scrollTo(0, scrollData[id])
      });
    }
  }

  render() {
    // 父组件提供的props
    const {list, renderRow, renderSeparator, refreshing, hasMoreData, refresh, endRefresh, renderHeader, id, className} = this.props;
    // 定义dataSource
    const dataSource = new AntdListView.DataSource({
      rowHasChanged: (row1, row2) => row1 !== row2,
    });

    // 下拉刷新属性配置
    let refreshProps = {};
    if (refresh) {
      refreshProps = {
        pullToRefresh: <PullToRefresh refreshing={refreshing || this.state.refreshing}
                                      damping={damping}
                                      onRefresh={() => {
                                        setTimeout(() => {
                                          refresh();
                                        }, time)
                                      }}
        />
      }
    }

    // 上拉刷新属性配置
    let endRefreshProps = {};
    if (endRefresh) {
      endRefreshProps = {
        onEndReachedThreshold: onEndReachedThreshold,
        onEndReached: () => {
          endRefresh();
        },
        renderFooter: () => (<div className='zk_slight-tip'>
          {hasMoreData ? '' : '没有更多记录'}
        </div>)
      }
    }
    //间隔修饰
    let separator = (sectionID, rowID) => (
      <div
        key={`${sectionID}-${rowID}`}
        className={renderSeparator || 'zk_separator'}
      />
    );
    let isUseBody = this.props.useBodyScroll ? this.props.useBodyScroll : false;
    return (
      <AntdListView
        renderHeader={renderHeader}
        dataSource={dataSource.cloneWithRows(list)}
        renderRow={renderRow}
        renderSeparator={separator}
        useBodyScroll={isUseBody}
        style={{
          height: isUseBody ? document.documentElement.clientHeight : this.props.height,
          overflow: 'auto',
          width: '100%'
        }}
        className = {className}
        ref={id}
        pageSize={this.state.size}
        initialListSize={this.state.size}
        onScroll={(e) => this.onScroll(e)}
        scrollEventThrottle={scrollEventThrottle}
        {...refreshProps}
        {...endRefreshProps}

      >
      </AntdListView>
    )
  }
}

ListView.propTypes = {
  // 列表标识
  id: PropTypes.string.isRequired,
  // 容器高度
  height: PropTypes.number,
  // 是否使用 html 的 body 作为滚动容器
  useBodyScroll: PropTypes.bool,
  // 列表头部标题
  renderHeader: PropTypes.func,
  // 列表数据
  list: PropTypes.array.isRequired,
  //样式类名称
  className: PropTypes.string,
  // 返回每一条数列表的数据以及自定义样式
  renderRow: PropTypes.func.isRequired,
  // 每一条之间分割物
  renderSeparator: PropTypes.string,
  // 以下2个配合实现下拉刷新
  //下拉刷新回调函数
  refresh: PropTypes.func,
  //是否显示刷新状态
  refreshing: PropTypes.bool,
  // 以下2个配合实现上拉加载更多数据
  //是否有数据
  hasMoreData: PropTypes.bool.isRequired,
  //上拉回调函数
  endRefresh: PropTypes.func.isRequired,
};

export default ListView;
